-- {{{ HEADER: File headers
module Main where

import IO 
import System (getArgs)
import System.Exit

import LexTocc
import ParTocc
import PrintTocc
import AbsTocc
import LayoutTocc
import ErrM

import ToccOpt
import ToccCompile
import ToccProto
import ToccListVar
import ToccLib

-- }}}

-- {{{ FUNCTIONS: Checking function

check :: Bool -> [String] -> IO ()
check v [] = putStr ""
check v (file:fs) = do
  code <- readFile file
  let lex = resolveLayout True . myLexer $ code
  let cst = pProgram_spec lex
  case varchkProgram_spec cst of
    Nothing -> if v
      then putStr $ file ++ ": \n" ++ case cst of
        Ok tree -> printTree tree
        _ -> error("Internal error: This tree should be ok")
      else putStr $ file ++ ": ok\n"
    Just err -> hPutStrLn stderr $ "In file " ++ file ++ ":\n" ++ indent " " err
  check v fs


-- }}}

-- {{{ FUNCTIONS: Compilation functions

showCCode :: CCode -> IO ()
showCCode (CCode t s h) = putStr $ cCodeDefaultHeader ++ t ++ h ++ s


cCodeDefaultHeader :: String
cCodeDefaultHeader = "#include <iostream>\n"
  ++ "#include <vector>\n"
  ++ "#include <string>\n"
  ++ "#include \"ControlProc.hpp\"\n"
  ++ "#include \"ParProc.hpp\"\n"
  ++ "#include \"ParEndProc.hpp\"\n"
  ++ "#include \"CommProc.hpp\"\n"
  ++ "#include \"CodeProc.hpp\"\n"
  ++ "#include \"Channel.hpp\"\n"
  ++ "#include \"TimerProc.hpp\"\n"
  ++ "#include \"TimerEndProc.hpp\"\n"
  ++ "typedef int INT;\n"
  ++ "typedef bool BOOL;\n"
  ++ "typedef std::string STRING;\n"
  ++ "typedef double REAL;\n"
  ++ "\n"


compile :: Bool -> String -> String -> String -> IO ()
compile v makef outf file = do
  code <- readFile file
  let lex = resolveLayout True . myLexer $ code
  let cst = pProgram_spec lex
  case varchkProgram_spec cst of
    Just err -> do
      hPutStr stderr $ file ++ ":\n" ++ indent " " err
      exitWith (ExitFailure 2)
    Nothing -> case compileProgram_spec cst of 
      Ok cCode -> showCCode cCode
      Bad err -> do
        hPutStrLn stderr $ file ++ ":\n" ++ indent " " err
        exitWith (ExitFailure 2)

-- }}}

-- {{{ FUNCTIONS: Main

main :: IO ()
main = do 
  action <- getArgs >>= parseOpts
  case action of 
    ActionCheck v fs -> check v fs
    ActionCompile v makef outf fs -> case length fs of
      1 -> compile v makef outf (head fs)
      _ -> error("Not implemented: Compiling more than one file at once")

-- }}}
