-- {{{ HEADER: File headers
module ToccConstExp (evalConstExp) where

-- This file contains code that does compile-time calculations. It only does
-- integer calculations. 

import AbsTocc
import PrintTocc 
import List

import ToccLocalsDecl

-- }}}

-- {{{ FUNCTIONS: Helper functions
 
readInt :: String -> Integer
readInt str = read str


-- Call binary function with Value parameters and Value result
binValueFun :: (Integer -> Integer -> Integer) -> LocalValue -> LocalValue -> LocalValue
binValueFun fun lhs rhs = case (lhs, rhs) of
  (ValueInt a, ValueInt b) -> ValueInt $ fun a b
  (ValueUnknown, _) -> ValueUnknown
  (_, ValueUnknown) -> ValueUnknown
  _ -> ValueKnownExpandTime


-- Call unary function with Value parameters and Value result
unValueFun :: (Integer -> Integer) -> LocalValue -> LocalValue
unValueFun fun arg = case arg of
  ValueInt a -> ValueInt $ fun a
  _ -> arg


logAnd :: Integer -> Integer -> Integer
logAnd x y = if x /= 0 && y /= 0 then 1 else 0


logOr :: Integer -> Integer -> Integer
logOr x y = if x /= 0 || y /= 0 then 1 else 0


logCmp :: (Integer -> Integer -> Bool) -> Integer -> Integer -> Integer
logCmp fun x y = if x `fun` y then 1 else 0


leftShift :: Integer -> Integer -> Integer
leftShift x y = x * 2 ^ y


rightShift :: Integer -> Integer -> Integer
rightShift x y = x `div` 2 ^ y


-- }}}

-- {{{ FUNCTIONS: Evaluate constant expressions

constIntConstVal_spec :: ConstVal_spec -> LocalValue
constIntConstVal_spec x = case x of
  ConstInt tokint  -> ValueInt $ readInt (printTree tokint)
  _ -> ValueUnknown


constIntExp :: Locals -> Exp -> LocalValue
constIntExp locals x = case x of
  ExpPlus exp0 exp        -> binValueFun (+) (constIntExp locals exp0) (constIntExp locals exp)
  ExpMinus exp0 exp       -> binValueFun (-) (constIntExp locals exp0) (constIntExp locals exp)
  ExpTimes exp0 exp       -> binValueFun (*) (constIntExp locals exp0) (constIntExp locals exp)
  ExpDiv exp0 exp         -> binValueFun div (constIntExp locals exp0) (constIntExp locals exp)
  ExpMod exp0 exp         -> binValueFun rem (constIntExp locals exp0) (constIntExp locals exp)
  ExpNegative exp         -> unValueFun negate (constIntExp locals exp)
  ExpPositive exp         -> constIntExp locals exp
  ExpConst constval_spec  -> constIntConstVal_spec constval_spec
  ExpVar lowerident       -> case find (\x -> locName x == (printTree lowerident)) locals of
                               Nothing -> ValueUnknown
                               Just Local {locValue=v} -> v
  ExpLor exp0 exp         -> binValueFun  logAnd (constIntExp locals exp0) (constIntExp locals exp)
  ExpLand exp0 exp        -> binValueFun logOr (constIntExp locals exp0) (constIntExp locals exp)
  Expeq exp0 exp          -> binValueFun (logCmp (==)) (constIntExp locals exp0) (constIntExp locals exp)
  Expneq exp0 exp         -> binValueFun (logCmp (/=)) (constIntExp locals exp0) (constIntExp locals exp)
  Explthen exp0 exp       -> binValueFun (logCmp (<=)) (constIntExp locals exp0) (constIntExp locals exp)
  Expgrthen exp0 exp      -> binValueFun (logCmp (>=)) (constIntExp locals exp0) (constIntExp locals exp)
  Exple exp0 exp          -> binValueFun (logCmp (<=)) (constIntExp locals exp0) (constIntExp locals exp)
  Expge exp0 exp          -> binValueFun (logCmp (>=)) (constIntExp locals exp0) (constIntExp locals exp)
  ExpLeft exp0 exp        -> binValueFun leftShift (constIntExp locals exp0) (constIntExp locals exp)
  ExpRight exp0 exp       -> binValueFun rightShift (constIntExp locals exp0) (constIntExp locals exp)
  -- TODO: Implement these
  ExpBitor exp0 exp       -> ValueUnknown
  ExpBitexor exp0 exp     -> ValueUnknown
  ExpBitand exp0 exp      -> ValueUnknown
  _                       -> ValueUnknown


evalConstExp :: Locals -> Exp -> LocalValue
evalConstExp locals exp = constIntExp locals exp

-- }}}
