-- {{{ HEADER: File headers
module ToccLib where

import List
import Maybe

import AbsTocc

-- }}}
 
-- {{{ FUNCTIONS: Various

indent :: String -> String -> String
indent _ "" = ""
indent spaces s = if last s == '\n'
  then spaces ++ (concat $ intersperse ("\n" ++ spaces) $ lines $ init s) ++ "\n"
  else spaces ++ (concat $ intersperse ("\n" ++ spaces) $ lines $ s)


-- }}}

-- {{{ FUNCTIONS: Partition if/else/elseif procs, check for valid single statement

-- Given a sequence of elseif*/else?/other, return the pair ([elseif*/else?],[other]) 
-- For syntactic reasons if/elseif/else are treated as separate (and
-- independent) statements in the grammar. This separates a conditional from the
-- following statements.
partitionElseIfs :: [Proc_spec] -> ([Proc_spec], [Proc_spec])
partitionElseIfs [] = ([], [])
partitionElseIfs (p:ps) = case p of
  ProcElseIf _ _ -> ([p] ++ fst rest, snd rest)
  ProcElse _ -> ([p], ps)
  _ -> ([], p:ps)
  where
    rest = partitionElseIfs ps


-- Returns true for legal single statements (single procs including
-- if..elseif*..else?)
isSingleStatement :: [Proc_spec] -> Bool
isSingleStatement [] = error "isSingleStatement called with empty list"
isSingleStatement (x:xs) = case x of
  ProcIf _ _ -> snd (partitionElseIfs xs) == []
  _ -> xs == []


partitionProcs :: [Proc_spec] -> [[Proc_spec]]
partitionProcs [] = []
partitionProcs (x:xs) = case x of
  ProcIf _ _ -> (x: (fst $ partitionElseIfs xs)): (partitionProcs $ snd $ partitionElseIfs xs)
  _ -> [x]: partitionProcs xs


-- }}}

