-- {{{ HEADER: File headers
module ToccProto (lookupProto) where

-- This file contains code that finds the prototypes for all parametric
-- processes in a program, and also functions for looking up those prototypes.
-- (F.ex if you want to know if the third parameter in process `Proc' is passed
-- by value or by reference)

import AbsTocc
import PrintTocc
import ErrM
import List
import Maybe

import ToccLocals
import ToccLocalsDecl

-- }}}
 
-- {{{ TYPE: Proto  = (String, Locals) 

type Proto = (String, Locals)

-- }}} 

-- {{{ FUNCTIONS: Search through program for named process prototypes.

protoProgram_spec :: Program_spec -> [Proto]
protoProgram_spec x = case x of
  Program global_specs  -> catMaybes $ map protoGlobal_spec global_specs


protoGlobal_spec :: Global_spec -> Maybe Proto
protoGlobal_spec x = case x of
  GlobalProcess capident param_specs block_spec  -> 
    Just (printTree capident, localsParam_specs param_specs)
  _ -> Nothing


-- }}}

-- {{{ FUNCTIONS: Top level functions lookupProto

-- Given a program, a named process, and a parameter number, return a Local for
-- that parameter.
lookupProto :: Program_spec -> String -> Int -> Err Local
lookupProto ps proc paramIx = case lookup proc (protoProgram_spec ps) of
  Nothing -> Bad $ "Process `" ++ proc ++ "' not found"
  Just args -> if paramIx `elem` [1..length args]
    then Ok $ args !! (paramIx-1)
    else Bad $ "Process `" ++ proc ++ "' has no param #" ++ show paramIx ++ "\n"


--}}}
