module PrintTocc where

-- pretty-printer generated by the BNF converter

import AbsTocc
import Char

-- the top-level printing method
printTree :: Print a => a -> String
printTree = render . prt 0

type Doc = [ShowS] -> [ShowS]

doc :: ShowS -> Doc
doc = (:)

render :: Doc -> String
render d = rend 0 (map ($ "") $ d []) "" where
  rend i ss = case ss of
    "["      :ts -> showChar '[' . rend i ts
    "("      :ts -> showChar '(' . rend i ts
    "{"      :ts -> showChar '{' . new (i+1) . rend (i+1) ts
    "}" : ";":ts -> new (i-1) . space "}" . showChar ';' . new (i-1) . rend (i-1) ts
    "}"      :ts -> new (i-1) . showChar '}' . new (i-1) . rend (i-1) ts
    ";"      :ts -> showChar ';' . new i . rend i ts
    t  : "," :ts -> showString t . space "," . rend i ts
    t  : ")" :ts -> showString t . showChar ')' . rend i ts
    t  : "]" :ts -> showString t . showChar ']' . rend i ts
    t        :ts -> space t . rend i ts
    _            -> id
  new i   = showChar '\n' . replicateS (2*i) (showChar ' ') . dropWhile isSpace
  space t = showString t . (\s -> if null s then "" else (' ':s))

parenth :: Doc -> Doc
parenth ss = doc (showChar '(') . ss . doc (showChar ')')

concatS :: [ShowS] -> ShowS
concatS = foldr (.) id

concatD :: [Doc] -> Doc
concatD = foldr (.) id

replicateS :: Int -> ShowS -> ShowS
replicateS n f = concatS (replicate n f)

-- the printer class does the job
class Print a where
  prt :: Int -> a -> Doc
  prtList :: [a] -> Doc
  prtList = concatD . map (prt 0)

instance Print a => Print [a] where
  prt _ = prtList

instance Print Char where
  prt _ s = doc (showChar '\'' . mkEsc '\'' s . showChar '\'')
  prtList s = doc (showChar '"' . concatS (map (mkEsc '"') s) . showChar '"')

mkEsc :: Char -> Char -> ShowS
mkEsc q s = case s of
  _ | s == q -> showChar '\\' . showChar s
  '\\'-> showString "\\\\"
  '\n' -> showString "\\n"
  '\t' -> showString "\\t"
  _ -> showChar s

prPrec :: Int -> Int -> Doc -> Doc
prPrec i j = if j<i then parenth else id


instance Print Integer where
  prt _ x = doc (shows x)


instance Print Double where
  prt _ x = doc (shows x)



instance Print CapIdent where
  prt _ (CapIdent i) = doc (showString i)


instance Print LowerIdent where
  prt _ (LowerIdent i) = doc (showString i)


instance Print UpperIdent where
  prt _ (UpperIdent i) = doc (showString i)



instance Print Init_spec where
  prt i e = case e of
   InitExp exp -> prPrec i 0 (concatD [doc (showString "=") , prt 1 exp])
   InitNone  -> prPrec i 0 (concatD [])


instance Print Decl_spec where
  prt i e = case e of
   Decl qualifier_spec type_spec array_spec lowerident -> prPrec i 0 (concatD [prt 0 qualifier_spec , prt 0 type_spec , prt 0 array_spec , prt 0 lowerident])


instance Print Array_spec where
  prt i e = case e of
   ArraySized exp -> prPrec i 0 (concatD [doc (showString "[") , prt 1 exp , doc (showString "]")])
   ArrayUnknown  -> prPrec i 0 (concatD [doc (showString "[") , doc (showString "]")])
   ArrayNone  -> prPrec i 0 (concatD [])


instance Print Variable_spec where
  prt i e = case e of
   Var decl_spec init_spec -> prPrec i 0 (concatD [prt 0 decl_spec , prt 0 init_spec])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Param_spec where
  prt i e = case e of
   Param decl_spec -> prPrec i 0 (concatD [prt 0 decl_spec])

  prtList es = case es of
   [] -> (concatD [])
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ",") , prt 0 xs])

instance Print Qualifier_spec where
  prt i e = case e of
   QualConst  -> prPrec i 0 (concatD [doc (showString "CONST")])
   QualChannel  -> prPrec i 0 (concatD [doc (showString "CHAN")])
   QualChannelIn  -> prPrec i 0 (concatD [doc (showString "IN")])
   QualChannelOut  -> prPrec i 0 (concatD [doc (showString "OUT")])
   QualByVal  -> prPrec i 0 (concatD [doc (showString "VAL")])
   QualReadWrite  -> prPrec i 0 (concatD [])


instance Print Type_spec where
  prt i e = case e of
   TypeString  -> prPrec i 0 (concatD [doc (showString "STRING")])
   TypeBool  -> prPrec i 0 (concatD [doc (showString "BOOL")])
   TypeInt  -> prPrec i 0 (concatD [doc (showString "INT")])
   TypeFloat  -> prPrec i 0 (concatD [doc (showString "REAL")])
   TypeCustom upperident -> prPrec i 0 (concatD [prt 0 upperident])


instance Print Program_spec where
  prt i e = case e of
   Program global_specs -> prPrec i 0 (concatD [prt 0 global_specs])


instance Print Global_spec where
  prt i e = case e of
   GlobalProcess capident param_specs singlestblock_spec -> prPrec i 0 (concatD [doc (showString "PROC") , prt 0 capident , doc (showString "(") , prt 0 param_specs , doc (showString ")") , doc (showString ":") , prt 0 singlestblock_spec])
   GlobalTypedef upperident type_spec array_spec -> prPrec i 0 (concatD [doc (showString "TYPE") , prt 0 upperident , doc (showString "=") , prt 0 type_spec , prt 0 array_spec])
   GlobalInclude str -> prPrec i 0 (concatD [doc (showString "CINCLUDE") , prt 0 str])
   GlobalSInclude str -> prPrec i 0 (concatD [doc (showString "CSYSINCLUDE") , prt 0 str])

  prtList es = case es of
   [] -> (concatD [])
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Block_spec where
  prt i e = case e of
   Block variable_specs proc_specs -> prPrec i 0 (concatD [doc (showString "{") , prt 0 variable_specs , prt 0 proc_specs , doc (showString "}")])


instance Print SingleStBlock_spec where
  prt i e = case e of
   SingleStBlock variable_specs proc_specs -> prPrec i 0 (concatD [doc (showString "{") , prt 0 variable_specs , prt 0 proc_specs , doc (showString "}")])


instance Print Order_spec where
  prt i e = case e of
   OrderSeq  -> prPrec i 0 (concatD [doc (showString "SEQ")])
   OrderPar  -> prPrec i 0 (concatD [doc (showString "PAR")])


instance Print Proc_spec where
  prt i e = case e of
   ProcAlt alt_spec -> prPrec i 0 (concatD [prt 0 alt_spec])
   ProcAssign lowerident assignment_op exp -> prPrec i 0 (concatD [prt 0 lowerident , prt 0 assignment_op , prt 1 exp])
   ProcAssignArr lowerident exp0 assignment_op exp -> prPrec i 0 (concatD [prt 0 lowerident , doc (showString "[") , prt 1 exp0 , doc (showString "]") , prt 0 assignment_op , prt 1 exp])
   ProcCall capident arg_specs -> prPrec i 0 (concatD [prt 0 capident , doc (showString "(") , prt 0 arg_specs , doc (showString ")")])
   ProcComm comm_spec -> prPrec i 0 (concatD [prt 0 comm_spec])
   ProcDelay exp -> prPrec i 0 (concatD [doc (showString "DELAY") , prt 1 exp])
   ProcElse singlestblock_spec -> prPrec i 0 (concatD [doc (showString "ELSE") , doc (showString ":") , prt 0 singlestblock_spec])
   ProcElseIf exp singlestblock_spec -> prPrec i 0 (concatD [doc (showString "ELSE") , doc (showString "IF") , prt 1 exp , doc (showString ":") , prt 0 singlestblock_spec])
   ProcExp exp -> prPrec i 0 (concatD [prt 1 exp])
   ProcFor order_spec lowerident exp0 exp singlestblock_spec -> prPrec i 0 (concatD [prt 0 order_spec , doc (showString "FOR") , prt 0 lowerident , doc (showString "=") , prt 1 exp0 , doc (showString "TO") , prt 1 exp , doc (showString ":") , prt 0 singlestblock_spec])
   ProcIf exp singlestblock_spec -> prPrec i 0 (concatD [doc (showString "IF") , prt 1 exp , doc (showString ":") , prt 0 singlestblock_spec])
   ProcOrder order_spec block_spec -> prPrec i 0 (concatD [prt 0 order_spec , doc (showString ":") , prt 0 block_spec])
   ProcPrint exps -> prPrec i 0 (concatD [doc (showString "PRINT") , prt 1 exps])
   ProcSkip  -> prPrec i 0 (concatD [doc (showString "SKIP")])
   ProcStop  -> prPrec i 0 (concatD [doc (showString "STOP")])
   ProcTime exp singlestblock_spec -> prPrec i 0 (concatD [doc (showString "TIME") , prt 1 exp , doc (showString ":") , prt 0 singlestblock_spec])
   ProcWhile exp singlestblock_spec -> prPrec i 0 (concatD [doc (showString "WHILE") , prt 1 exp , doc (showString ":") , prt 0 singlestblock_spec])

  prtList es = case es of
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Arg_spec where
  prt i e = case e of
   Arg exp -> prPrec i 0 (concatD [prt 1 exp])

  prtList es = case es of
   [] -> (concatD [])
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ",") , prt 0 xs])

instance Print Comm_spec where
  prt i e = case e of
   CommSend lowerident0 lowerident -> prPrec i 0 (concatD [prt 0 lowerident0 , doc (showString "!") , prt 0 lowerident])
   CommRecv lowerident0 lowerident -> prPrec i 0 (concatD [prt 0 lowerident0 , doc (showString "?") , prt 0 lowerident])


instance Print Alt_spec where
  prt i e = case e of
   Alt altalt_specs -> prPrec i 0 (concatD [doc (showString "ALT") , doc (showString ":") , doc (showString "{") , prt 0 altalt_specs , doc (showString "}")])
   AltRepl lowerident exp0 exp altalt_specs -> prPrec i 0 (concatD [doc (showString "ALT") , doc (showString "FOR") , prt 0 lowerident , doc (showString "=") , prt 1 exp0 , doc (showString "TO") , prt 1 exp , doc (showString ":") , doc (showString "{") , prt 0 altalt_specs , doc (showString "}")])


instance Print AltComm_spec where
  prt i e = case e of
   AltCommRecv lowerident0 lowerident -> prPrec i 0 (concatD [prt 0 lowerident0 , doc (showString "?") , prt 0 lowerident])


instance Print AltAlt_spec where
  prt i e = case e of
   AltAltNested alt_spec -> prPrec i 0 (concatD [prt 0 alt_spec])
   AltAltGuard exp altcomm_spec singlestblock_spec -> prPrec i 0 (concatD [prt 1 exp , doc (showString "->") , prt 0 altcomm_spec , doc (showString ":") , prt 0 singlestblock_spec])
   AltAltSkip exp -> prPrec i 0 (concatD [prt 1 exp , doc (showString "->") , doc (showString "SKIP")])
   AltAlt altcomm_spec singlestblock_spec -> prPrec i 0 (concatD [prt 0 altcomm_spec , doc (showString ":") , prt 0 singlestblock_spec])

  prtList es = case es of
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Assignment_op where
  prt i e = case e of
   Assign  -> prPrec i 0 (concatD [doc (showString "=")])
   AssignMul  -> prPrec i 0 (concatD [doc (showString "*=")])
   AssignDiv  -> prPrec i 0 (concatD [doc (showString "/=")])
   AssignMod  -> prPrec i 0 (concatD [doc (showString "%=")])
   AssignAdd  -> prPrec i 0 (concatD [doc (showString "+=")])
   AssignSub  -> prPrec i 0 (concatD [doc (showString "-=")])
   AssignLeft  -> prPrec i 0 (concatD [doc (showString "<<=")])
   AssignRight  -> prPrec i 0 (concatD [doc (showString ">>=")])
   AssignAnd  -> prPrec i 0 (concatD [doc (showString "&=")])
   AssignXor  -> prPrec i 0 (concatD [doc (showString "^=")])
   AssignOr  -> prPrec i 0 (concatD [doc (showString "|=")])


instance Print ConstVal_spec where
  prt i e = case e of
   ConstInt n -> prPrec i 0 (concatD [prt 0 n])
   ConstFloat d -> prPrec i 0 (concatD [prt 0 d])
   ConstString str -> prPrec i 0 (concatD [prt 0 str])
   ConstTrue  -> prPrec i 0 (concatD [doc (showString "true")])
   ConstFalse  -> prPrec i 0 (concatD [doc (showString "false")])
   ConstChar c -> prPrec i 0 (concatD [prt 0 c])


instance Print Exp where
  prt i e = case e of
   ExpLor exp0 exp -> prPrec i 1 (concatD [prt 1 exp0 , doc (showString "||") , prt 2 exp])
   ExpLand exp0 exp -> prPrec i 2 (concatD [prt 2 exp0 , doc (showString "&&") , prt 3 exp])
   Expeq exp0 exp -> prPrec i 3 (concatD [prt 3 exp0 , doc (showString "==") , prt 4 exp])
   Expneq exp0 exp -> prPrec i 3 (concatD [prt 3 exp0 , doc (showString "!=") , prt 4 exp])
   Explthen exp0 exp -> prPrec i 4 (concatD [prt 4 exp0 , doc (showString "<") , prt 5 exp])
   Expgrthen exp0 exp -> prPrec i 4 (concatD [prt 4 exp0 , doc (showString ">") , prt 5 exp])
   Exple exp0 exp -> prPrec i 4 (concatD [prt 4 exp0 , doc (showString "<=") , prt 5 exp])
   Expge exp0 exp -> prPrec i 4 (concatD [prt 4 exp0 , doc (showString ">=") , prt 5 exp])
   ExpBitor exp0 exp -> prPrec i 5 (concatD [prt 5 exp0 , doc (showString "|") , prt 6 exp])
   ExpBitexor exp0 exp -> prPrec i 6 (concatD [prt 6 exp0 , doc (showString "^") , prt 7 exp])
   ExpBitand exp0 exp -> prPrec i 7 (concatD [prt 7 exp0 , doc (showString "&") , prt 8 exp])
   ExpLeft exp0 exp -> prPrec i 8 (concatD [prt 8 exp0 , doc (showString "<<") , prt 9 exp])
   ExpRight exp0 exp -> prPrec i 8 (concatD [prt 8 exp0 , doc (showString ">>") , prt 9 exp])
   ExpPlus exp0 exp -> prPrec i 9 (concatD [prt 9 exp0 , doc (showString "+") , prt 10 exp])
   ExpMinus exp0 exp -> prPrec i 9 (concatD [prt 9 exp0 , doc (showString "-") , prt 10 exp])
   ExpTimes exp0 exp -> prPrec i 10 (concatD [prt 10 exp0 , doc (showString "*") , prt 11 exp])
   ExpDiv exp0 exp -> prPrec i 10 (concatD [prt 10 exp0 , doc (showString "/") , prt 11 exp])
   ExpMod exp0 exp -> prPrec i 10 (concatD [prt 10 exp0 , doc (showString "%") , prt 11 exp])
   ExpNegative exp -> prPrec i 11 (concatD [doc (showString "-") , prt 11 exp])
   ExpPositive exp -> prPrec i 11 (concatD [doc (showString "+") , prt 11 exp])
   ExpIndex lowerident exp -> prPrec i 12 (concatD [prt 0 lowerident , doc (showString "[") , prt 1 exp , doc (showString "]")])
   ExpFunc lowerident exps -> prPrec i 12 (concatD [prt 0 lowerident , doc (showString "(") , prt 1 exps , doc (showString ")")])
   ExpConst constval_spec -> prPrec i 13 (concatD [prt 0 constval_spec])
   ExpVar lowerident -> prPrec i 13 (concatD [prt 0 lowerident])

  prtList es = case es of
   [] -> (concatD [])
   [x] -> (concatD [prt 1 x])
   x:xs -> (concatD [prt 1 x , doc (showString ",") , prt 1 xs])


