#include <iostream>
#include <vector>
#include <string>
#include "ControlProc.hpp"
#include "ParProc.hpp"
#include "ParEndProc.hpp"
#include "CommProc.hpp"
#include "CodeProc.hpp"
#include "Channel.hpp"
#include "TimerProc.hpp"
#include "TimerEndProc.hpp"
typedef int INT;
typedef bool BOOL;
typedef std::string STRING;
typedef double REAL;

// {{{ Headers for process `Main'
// {{{ Class Main : ControlProc (Process)
class Main : public ControlProc {
	private:
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main\\nProcess";}
		Main(Proc* pop) :
			ControlProc(pop) {}
};
// }}}
// {{{ Class Main_1 : ControlProc (New Scope)
class Main_1 : public ControlProc {
	private:
		const int* n;
		ChannelVector<int>* chans;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1\\nNew Scope";}
		Main_1(Proc* pop) :
			ControlProc(pop),
			n(new const int(3)),
			chans(new ChannelVector<int>((2*(*n)))) {}
};
// }}}
// {{{ Class Main_1_2 : CodeProc (Delete Scope)
class Main_1_2 : public CodeProc {
	private:
		const int* n;
		ChannelVector<int>* chans;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Main_1_2\\nDelete Scope";}
		Main_1_2(Proc* pop, const int* P_n, ChannelVector<int>* P_chans) :
			CodeProc(pop),
			n(P_n),
			chans(P_chans) {}
};
// }}}
// {{{ Class Main_1_1 : ParProc (PAR FOR)
class Main_1_1 : public ParProc {
	private:
		const int* n;
		ChannelVector<int>* chans;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Main_1_1\\nPAR FOR";}
		Main_1_1(Proc* pop, const int* P_n, ChannelVector<int>* P_chans) :
			ParProc(pop),
			n(P_n),
			chans(P_chans) {}
};
// }}}
// {{{ Class Main_1_1_1 : ParProc (PAR)
class Main_1_1_1 : public ParProc {
	private:
		const int* n;
		ChannelVector<int>* chans;
		int i;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Main_1_1_1\\nPAR";}
		Main_1_1_1(Proc* pop, const int* P_n, ChannelVector<int>* P_chans, int P_i) :
			ParProc(pop),
			n(P_n),
			chans(P_chans),
			i(P_i) {}
};
// }}}
// {{{ Class Main_1_1_1_1 : ControlProc (Call)
class Main_1_1_1_1 : public ControlProc {
	private:
		const int* n;
		std::vector<SendChannel<int> *>* chans;
		int i;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1_1_1_1\\nCall";}
		Main_1_1_1_1(Proc* pop, const int* P_n, std::vector<SendChannel<int> *>* P_chans, int P_i) :
			ControlProc(pop),
			n(P_n),
			chans(P_chans),
			i(P_i) {}
};
// }}}
// {{{ Class Main_1_1_1_2 : ControlProc (Call)
class Main_1_1_1_2 : public ControlProc {
	private:
		std::vector<ReceiveChannel<int> *>* chans;
		int i;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1_1_1_2\\nCall";}
		Main_1_1_1_2(Proc* pop, std::vector<ReceiveChannel<int> *>* P_chans, int P_i) :
			ControlProc(pop),
			chans(P_chans),
			i(P_i) {}
};
// }}}
// }}}

// {{{ Headers for process `Fork'
// {{{ Class Fork : ControlProc (Process)
class Fork : public ControlProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Fork\\nProcess";}
		Fork(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil) :
			ControlProc(pop),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)) {}
};
// }}}
// {{{ Class Fork_1 : ControlProc (New Scope)
class Fork_1 : public ControlProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Fork_1\\nNew Scope";}
		Fork_1(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil) :
			ControlProc(pop),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(new int()) {}
};
// }}}
// {{{ Class Fork_1_2 : CodeProc (Delete Scope)
class Fork_1_2 : public CodeProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Fork_1_2\\nDelete Scope";}
		Fork_1_2(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			CodeProc(pop),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Fork_1_1 : ControlProc (While)
class Fork_1_1 : public ControlProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Fork_1_1\\nWhile";}
		Fork_1_1(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			ControlProc(pop),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Fork_1_1_1 : ControlProc (ALT)
class Fork_1_1_1 : public ControlProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Fork_1_1_1\\nALT";}
		Fork_1_1_1(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			ControlProc(pop),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Fork_1_1_1_1 : CommProc (Comm)
class Fork_1_1_1_1 : public CommProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Fork_1_1_1_1\\nComm";}
		Fork_1_1_1_1(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			CommProc(pop, P_leftPhil, P_dummy),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Fork_1_1_1_1_1 : CommProc (Comm)
class Fork_1_1_1_1_1 : public CommProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void transfer();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Fork_1_1_1_1_1\\nComm";}
		Fork_1_1_1_1_1(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			CommProc(pop, P_leftPhil, P_dummy),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Fork_1_1_1_2 : CommProc (Comm)
class Fork_1_1_1_2 : public CommProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Fork_1_1_1_2\\nComm";}
		Fork_1_1_1_2(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			CommProc(pop, P_rightPhil, P_dummy),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Fork_1_1_1_2_1 : CommProc (Comm)
class Fork_1_1_1_2_1 : public CommProc {
	private:
		ReceiveChannel<int> * leftPhil;
		ReceiveChannel<int> * rightPhil;
		int* dummy;
	public:
		virtual void transfer();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Fork_1_1_1_2_1\\nComm";}
		Fork_1_1_1_2_1(Proc* pop, ReceiveChannel<int> * P_leftPhil, ReceiveChannel<int> * P_rightPhil, int* P_dummy) :
			CommProc(pop, P_rightPhil, P_dummy),
			leftPhil(take(P_leftPhil)),
			rightPhil(take(P_rightPhil)),
			dummy(P_dummy) {}
};
// }}}
// }}}

// {{{ Headers for process `Phil'
// {{{ Class Phil : ControlProc (Process)
class Phil : public ControlProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Phil\\nProcess";}
		Phil(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork) :
			ControlProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)) {}
};
// }}}
// {{{ Class Phil_1 : ControlProc (New Scope)
class Phil_1 : public ControlProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Phil_1\\nNew Scope";}
		Phil_1(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork) :
			ControlProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(new int(0)) {}
};
// }}}
// {{{ Class Phil_1_2 : CodeProc (Delete Scope)
class Phil_1_2 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_2\\nDelete Scope";}
		Phil_1_2(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1 : ControlProc (Timer)
class Phil_1_1 : public ControlProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Phil_1_1\\nTimer";}
		Phil_1_1(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			ControlProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1 : CodeProc (PRINT)
class Phil_1_1_1 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1\\nPRINT";}
		Phil_1_1_1(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_2 : CommProc (Comm)
class Phil_1_1_1_2 : public CommProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_2\\nComm";}
		Phil_1_1_1_2(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CommProc(pop, P_leftFork, P_dummy),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_3 : CodeProc (PRINT)
class Phil_1_1_1_3 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_3\\nPRINT";}
		Phil_1_1_1_3(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_4 : CommProc (Comm)
class Phil_1_1_1_4 : public CommProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_4\\nComm";}
		Phil_1_1_1_4(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CommProc(pop, P_rightFork, P_dummy),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_5 : CodeProc (PRINT)
class Phil_1_1_1_5 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_5\\nPRINT";}
		Phil_1_1_1_5(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_6 : CodeProc (DELAY)
class Phil_1_1_1_6 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 10; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_6\\nDELAY";}
		Phil_1_1_1_6(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_7 : CommProc (Comm)
class Phil_1_1_1_7 : public CommProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_7\\nComm";}
		Phil_1_1_1_7(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CommProc(pop, P_rightFork, P_dummy),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_8 : CodeProc (PRINT)
class Phil_1_1_1_8 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_8\\nPRINT";}
		Phil_1_1_1_8(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_9 : CommProc (Comm)
class Phil_1_1_1_9 : public CommProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_9\\nComm";}
		Phil_1_1_1_9(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CommProc(pop, P_leftFork, P_dummy),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// {{{ Class Phil_1_1_1_10 : CodeProc (PRINT)
class Phil_1_1_1_10 : public CodeProc {
	private:
		int philNo;
		SendChannel<int> * leftFork;
		SendChannel<int> * rightFork;
		int* dummy;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Phil_1_1_1_10\\nPRINT";}
		Phil_1_1_1_10(Proc* pop, int P_philNo, SendChannel<int> * P_leftFork, SendChannel<int> * P_rightFork, int* P_dummy) :
			CodeProc(pop),
			philNo(P_philNo),
			leftFork(take(P_leftFork)),
			rightFork(take(P_rightFork)),
			dummy(P_dummy) {}
};
// }}}
// }}}

// {{{ Source for process `Main'
// {{{ Function `void Main::expand()' (Process)
void Main::expand() {
	connect(add(new Main_1(pop)));
}
// }}}
// {{{ Function `void Main_1::expand()' (New Scope)
void Main_1::expand() {
	Proc* pop = add(new Main_1_2(this->pop, n, chans));
	connect(add(new Main_1_1(pop, n, chans)));
}
// }}}
// {{{ Function `void Main_1_2::run()' (Delete Scope)
void Main_1_2::run() {
	delete n;
	delete chans;
}
// }}}
// {{{ Function `void Main_1_1::expand()' (PAR FOR)
void Main_1_1::expand() {
	ParEndProc* join = add(new ParEndProc(pop, this));
	int i;
	for (i = 0; i <= ((*n)-1); (i)++) {
		connect(add(new Main_1_1_1(join, n, chans, i)));
	}
}
// }}}
// {{{ Function `void Main_1_1_1::expand()' (PAR)
void Main_1_1_1::expand() {
	ParEndProc* join = add(new ParEndProc(pop, this));
	connect(add(new Main_1_1_1_1(join, n, makeSend(chans), i)));
	connect(add(new Main_1_1_1_2(join, makeReceive(chans), i)));
}
// }}}
// {{{ Function `void Main_1_1_1_1::expand()' (Call)
void Main_1_1_1_1::expand() {
	connect(add(new Phil(pop, i, (*chans)[(2*i)], (*chans)[((((2*i)+(2*(*n)))-1)%(2*(*n)))])));
}
// }}}
// {{{ Function `void Main_1_1_1_2::expand()' (Call)
void Main_1_1_1_2::expand() {
	connect(add(new Fork(pop, (*chans)[(2*i)], (*chans)[((2*i)+1)])));
}
// }}}
// }}}

// {{{ Source for process `Fork'
// {{{ Function `void Fork::expand()' (Process)
void Fork::expand() {
	connect(add(new Fork_1(pop, leftPhil, rightPhil)));
}
// }}}
// {{{ Function `void Fork_1::expand()' (New Scope)
void Fork_1::expand() {
	Proc* pop = add(new Fork_1_2(this->pop, leftPhil, rightPhil, dummy));
	connect(add(new Fork_1_1(pop, leftPhil, rightPhil, dummy)));
}
// }}}
// {{{ Function `void Fork_1_2::run()' (Delete Scope)
void Fork_1_2::run() {
	delete dummy;
}
// }}}
// {{{ Function `void Fork_1_1::expand()' (While)
void Fork_1_1::expand() {
	if (1) {
		Proc* loop = add(new Fork_1_1(pop, leftPhil, rightPhil, dummy));
		connect(add(new Fork_1_1_1(loop, leftPhil, rightPhil, dummy)));
	}
	else {
		connect(pop);
	}
}
// }}}
// {{{ Function `void Fork_1_1_1::expand()' (ALT)
void Fork_1_1_1::expand() {
	for (int _phase = 0; _phase <= 2; _phase++) {
		if (_phase == 2 || leftPhil->getChannelBase()->isSenderReady()) {
			connect(add(new Fork_1_1_1_1(pop, leftPhil, rightPhil, dummy)));
			return;
		}
		if (_phase == 2 || rightPhil->getChannelBase()->isSenderReady()) {
			connect(add(new Fork_1_1_1_2(pop, leftPhil, rightPhil, dummy)));
			return;
		}
	}
}
// }}}
// {{{ Function `void Fork_1_1_1_1::expand()' (Comm)
void Fork_1_1_1_1::expand() {
	connect(add(new Fork_1_1_1_1_1(pop, leftPhil, rightPhil, dummy)));
}
// }}}
// {{{ Function `void Fork_1_1_1_1::transfer()' (Comm)
void Fork_1_1_1_1::transfer() {
	leftPhil->transfer();
}
// }}}
// {{{ Function `void Fork_1_1_1_1_1::transfer()' (Comm)
void Fork_1_1_1_1_1::transfer() {
	leftPhil->transfer();
}
// }}}
// {{{ Function `void Fork_1_1_1_2::expand()' (Comm)
void Fork_1_1_1_2::expand() {
	connect(add(new Fork_1_1_1_2_1(pop, leftPhil, rightPhil, dummy)));
}
// }}}
// {{{ Function `void Fork_1_1_1_2::transfer()' (Comm)
void Fork_1_1_1_2::transfer() {
	rightPhil->transfer();
}
// }}}
// {{{ Function `void Fork_1_1_1_2_1::transfer()' (Comm)
void Fork_1_1_1_2_1::transfer() {
	rightPhil->transfer();
}
// }}}
// }}}

// {{{ Source for process `Phil'
// {{{ Function `void Phil::expand()' (Process)
void Phil::expand() {
	connect(add(new Phil_1(pop, philNo, leftFork, rightFork)));
}
// }}}
// {{{ Function `void Phil_1::expand()' (New Scope)
void Phil_1::expand() {
	Proc* pop = add(new Phil_1_2(this->pop, philNo, leftFork, rightFork, dummy));
	connect(add(new Phil_1_1(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_2::run()' (Delete Scope)
void Phil_1_2::run() {
	delete dummy;
}
// }}}
// {{{ Function `void Phil_1_1::expand()' (Timer)
void Phil_1_1::expand() {
	TimerEndProc* pop = add(new TimerEndProc(this->pop));
	Proc* next = add(new Phil_1_1_1(pop, philNo, leftFork, rightFork, dummy));
	connect(add(new TimerProc(next, pop, 100)));
}
// }}}
// {{{ Function `void Phil_1_1_1::expand()' (PRINT)
void Phil_1_1_1::expand() {
	connect(add(new Phil_1_1_1_2(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1::run()' (PRINT)
void Phil_1_1_1::run() {
	std::cout << "Philosopher " << philNo << " wants to eat." << std::endl;
}
// }}}
// {{{ Function `void Phil_1_1_1_2::expand()' (Comm)
void Phil_1_1_1_2::expand() {
	connect(add(new Phil_1_1_1_3(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_2::transfer()' (Comm)
void Phil_1_1_1_2::transfer() {
	leftFork->transfer();
}
// }}}
// {{{ Function `void Phil_1_1_1_3::expand()' (PRINT)
void Phil_1_1_1_3::expand() {
	connect(add(new Phil_1_1_1_4(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_3::run()' (PRINT)
void Phil_1_1_1_3::run() {
	std::cout << "Philosopher " << philNo << " has got left fork." << std::endl;
}
// }}}
// {{{ Function `void Phil_1_1_1_4::expand()' (Comm)
void Phil_1_1_1_4::expand() {
	connect(add(new Phil_1_1_1_5(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_4::transfer()' (Comm)
void Phil_1_1_1_4::transfer() {
	rightFork->transfer();
}
// }}}
// {{{ Function `void Phil_1_1_1_5::expand()' (PRINT)
void Phil_1_1_1_5::expand() {
	connect(add(new Phil_1_1_1_6(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_5::run()' (PRINT)
void Phil_1_1_1_5::run() {
	std::cout << "Philosopher " << philNo << " has got both forks, eating." << std::endl;
}
// }}}
// {{{ Function `void Phil_1_1_1_6::expand()' (DELAY)
void Phil_1_1_1_6::expand() {
	connect(add(new Phil_1_1_1_7(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_6::run()' (DELAY)
void Phil_1_1_1_6::run() {
	usleep(10);
}
// }}}
// {{{ Function `void Phil_1_1_1_7::expand()' (Comm)
void Phil_1_1_1_7::expand() {
	connect(add(new Phil_1_1_1_8(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_7::transfer()' (Comm)
void Phil_1_1_1_7::transfer() {
	rightFork->transfer();
}
// }}}
// {{{ Function `void Phil_1_1_1_8::expand()' (PRINT)
void Phil_1_1_1_8::expand() {
	connect(add(new Phil_1_1_1_9(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_8::run()' (PRINT)
void Phil_1_1_1_8::run() {
	std::cout << "Philosopher " << philNo << " has given away right fork." << std::endl;
}
// }}}
// {{{ Function `void Phil_1_1_1_9::expand()' (Comm)
void Phil_1_1_1_9::expand() {
	connect(add(new Phil_1_1_1_10(pop, philNo, leftFork, rightFork, dummy)));
}
// }}}
// {{{ Function `void Phil_1_1_1_9::transfer()' (Comm)
void Phil_1_1_1_9::transfer() {
	leftFork->transfer();
}
// }}}
// {{{ Function `void Phil_1_1_1_10::run()' (PRINT)
void Phil_1_1_1_10::run() {
	std::cout << "Philosopher " << philNo << " has no forks." << std::endl;
}
// }}}
// }}}

