#include <iostream>
#include <vector>
#include <string>
#include "ControlProc.hpp"
#include "ParProc.hpp"
#include "ParEndProc.hpp"
#include "CommProc.hpp"
#include "CodeProc.hpp"
#include "Channel.hpp"
#include "TimerProc.hpp"
#include "TimerEndProc.hpp"
typedef int INT;
typedef bool BOOL;
typedef std::string STRING;
typedef double REAL;


#include "dummyfuncs.hpp"
typedef INT CODEDDATA ;
typedef INT DACBUFFER ;
// {{{ Headers for process `DiskRead'
// {{{ Class DiskRead : ControlProc (Process)
class DiskRead : public ControlProc {
	private:
		SendChannel<CODEDDATA> * ch;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "DiskRead\\nProcess";}
		DiskRead(Proc* pop, SendChannel<CODEDDATA> * P_ch) :
			ControlProc(pop),
			ch(take(P_ch)) {}
};
// }}}
// {{{ Class DiskRead_1 : ControlProc (New Scope)
class DiskRead_1 : public ControlProc {
	private:
		SendChannel<CODEDDATA> * ch;
		CODEDDATA* buff;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "DiskRead_1\\nNew Scope";}
		DiskRead_1(Proc* pop, SendChannel<CODEDDATA> * P_ch) :
			ControlProc(pop),
			ch(take(P_ch)),
			buff(new CODEDDATA()) {}
};
// }}}
// {{{ Class DiskRead_1_2 : CodeProc (Delete Scope)
class DiskRead_1_2 : public CodeProc {
	private:
		SendChannel<CODEDDATA> * ch;
		CODEDDATA* buff;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "DiskRead_1_2\\nDelete Scope";}
		DiskRead_1_2(Proc* pop, SendChannel<CODEDDATA> * P_ch, CODEDDATA* P_buff) :
			CodeProc(pop),
			ch(take(P_ch)),
			buff(P_buff) {}
};
// }}}
// {{{ Class DiskRead_1_1 : ControlProc (While)
class DiskRead_1_1 : public ControlProc {
	private:
		SendChannel<CODEDDATA> * ch;
		CODEDDATA* buff;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "DiskRead_1_1\\nWhile";}
		DiskRead_1_1(Proc* pop, SendChannel<CODEDDATA> * P_ch, CODEDDATA* P_buff) :
			ControlProc(pop),
			ch(take(P_ch)),
			buff(P_buff) {}
};
// }}}
// {{{ Class DiskRead_1_1_1 : CodeProc (Assign)
class DiskRead_1_1_1 : public CodeProc {
	private:
		SendChannel<CODEDDATA> * ch;
		CODEDDATA* buff;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "DiskRead_1_1_1\\nAssign";}
		DiskRead_1_1_1(Proc* pop, SendChannel<CODEDDATA> * P_ch, CODEDDATA* P_buff) :
			CodeProc(pop),
			ch(take(P_ch)),
			buff(P_buff) {}
};
// }}}
// {{{ Class DiskRead_1_1_1_2 : CommProc (Comm)
class DiskRead_1_1_1_2 : public CommProc {
	private:
		SendChannel<CODEDDATA> * ch;
		CODEDDATA* buff;
	public:
		virtual void transfer();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "DiskRead_1_1_1_2\\nComm";}
		DiskRead_1_1_1_2(Proc* pop, SendChannel<CODEDDATA> * P_ch, CODEDDATA* P_buff) :
			CommProc(pop, P_ch, P_buff),
			ch(take(P_ch)),
			buff(P_buff) {}
};
// }}}
// }}}

// {{{ Headers for process `Decode'
// {{{ Class Decode : ControlProc (Process)
class Decode : public ControlProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Decode\\nProcess";}
		Decode(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch) :
			ControlProc(pop),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)) {}
};
// }}}
// {{{ Class Decode_1 : ControlProc (New Scope)
class Decode_1 : public ControlProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
		CODEDDATA* coded;
		DACBUFFER* decoded;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Decode_1\\nNew Scope";}
		Decode_1(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch) :
			ControlProc(pop),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)),
			coded(new CODEDDATA()),
			decoded(new DACBUFFER()) {}
};
// }}}
// {{{ Class Decode_1_2 : CodeProc (Delete Scope)
class Decode_1_2 : public CodeProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
		CODEDDATA* coded;
		DACBUFFER* decoded;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Decode_1_2\\nDelete Scope";}
		Decode_1_2(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch, CODEDDATA* P_coded, DACBUFFER* P_decoded) :
			CodeProc(pop),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)),
			coded(P_coded),
			decoded(P_decoded) {}
};
// }}}
// {{{ Class Decode_1_1 : ControlProc (While)
class Decode_1_1 : public ControlProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
		CODEDDATA* coded;
		DACBUFFER* decoded;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Decode_1_1\\nWhile";}
		Decode_1_1(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch, CODEDDATA* P_coded, DACBUFFER* P_decoded) :
			ControlProc(pop),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)),
			coded(P_coded),
			decoded(P_decoded) {}
};
// }}}
// {{{ Class Decode_1_1_1 : CommProc (Comm)
class Decode_1_1_1 : public CommProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
		CODEDDATA* coded;
		DACBUFFER* decoded;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Decode_1_1_1\\nComm";}
		Decode_1_1_1(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch, CODEDDATA* P_coded, DACBUFFER* P_decoded) :
			CommProc(pop, P_codedch, P_coded),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)),
			coded(P_coded),
			decoded(P_decoded) {}
};
// }}}
// {{{ Class Decode_1_1_1_2 : CodeProc (Assign)
class Decode_1_1_1_2 : public CodeProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
		CODEDDATA* coded;
		DACBUFFER* decoded;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Decode_1_1_1_2\\nAssign";}
		Decode_1_1_1_2(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch, CODEDDATA* P_coded, DACBUFFER* P_decoded) :
			CodeProc(pop),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)),
			coded(P_coded),
			decoded(P_decoded) {}
};
// }}}
// {{{ Class Decode_1_1_1_3 : CommProc (Comm)
class Decode_1_1_1_3 : public CommProc {
	private:
		ReceiveChannel<CODEDDATA> * codedch;
		SendChannel<DACBUFFER> * decodedch;
		CODEDDATA* coded;
		DACBUFFER* decoded;
	public:
		virtual void transfer();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Decode_1_1_1_3\\nComm";}
		Decode_1_1_1_3(Proc* pop, ReceiveChannel<CODEDDATA> * P_codedch, SendChannel<DACBUFFER> * P_decodedch, CODEDDATA* P_coded, DACBUFFER* P_decoded) :
			CommProc(pop, P_decodedch, P_decoded),
			codedch(take(P_codedch)),
			decodedch(take(P_decodedch)),
			coded(P_coded),
			decoded(P_decoded) {}
};
// }}}
// }}}

// {{{ Headers for process `WriteDAC'
// {{{ Class WriteDAC : ControlProc (Process)
class WriteDAC : public ControlProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "WriteDAC\\nProcess";}
		WriteDAC(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop) :
			ControlProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class WriteDAC_1 : ControlProc (New Scope)
class WriteDAC_1 : public ControlProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "WriteDAC_1\\nNew Scope";}
		WriteDAC_1(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop) :
			ControlProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(new DACBUFFER()),
			cmd(new int()),
			play(new const int(1)) {}
};
// }}}
// {{{ Class WriteDAC_1_2 : CodeProc (Delete Scope)
class WriteDAC_1_2 : public CodeProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "WriteDAC_1_2\\nDelete Scope";}
		WriteDAC_1_2(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			CodeProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// {{{ Class WriteDAC_1_1 : ControlProc (While)
class WriteDAC_1_1 : public ControlProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "WriteDAC_1_1\\nWhile";}
		WriteDAC_1_1(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			ControlProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// {{{ Class WriteDAC_1_1_1 : ControlProc (Timer)
class WriteDAC_1_1_1 : public ControlProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "WriteDAC_1_1_1\\nTimer";}
		WriteDAC_1_1_1(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			ControlProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// {{{ Class WriteDAC_1_1_1_1 : CommProc (Comm)
class WriteDAC_1_1_1_1 : public CommProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "WriteDAC_1_1_1_1\\nComm";}
		WriteDAC_1_1_1_1(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			CommProc(pop, P_startstop, P_cmd),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// {{{ Class WriteDAC_1_1_1_1_2 : ControlProc (If)
class WriteDAC_1_1_1_1_2 : public ControlProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual void expand();
		virtual bool isUncertain() const {return true;}
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "WriteDAC_1_1_1_1_2\\nIf";}
		WriteDAC_1_1_1_1_2(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			ControlProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// {{{ Class WriteDAC_1_1_1_1_2_1 : CommProc (Comm)
class WriteDAC_1_1_1_1_2_1 : public CommProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual void transfer();
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "WriteDAC_1_1_1_1_2_1\\nComm";}
		WriteDAC_1_1_1_1_2_1(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			CommProc(pop, P_ch, P_decoded),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// {{{ Class WriteDAC_1_1_1_1_2_1_2 : CodeProc ((expr))
class WriteDAC_1_1_1_1_2_1_2 : public CodeProc {
	private:
		ReceiveChannel<DACBUFFER> * ch;
		ReceiveChannel<int> * startstop;
		DACBUFFER* decoded;
		int* cmd;
		const int* play;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "WriteDAC_1_1_1_1_2_1_2\\n(expr)";}
		WriteDAC_1_1_1_1_2_1_2(Proc* pop, ReceiveChannel<DACBUFFER> * P_ch, ReceiveChannel<int> * P_startstop, DACBUFFER* P_decoded, int* P_cmd, const int* P_play) :
			CodeProc(pop),
			ch(take(P_ch)),
			startstop(take(P_startstop)),
			decoded(P_decoded),
			cmd(P_cmd),
			play(P_play) {}
};
// }}}
// }}}

// {{{ Headers for process `UserIface'
// {{{ Class UserIface : ControlProc (Process)
class UserIface : public ControlProc {
	private:
		SendChannel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "UserIface\\nProcess";}
		UserIface(Proc* pop, SendChannel<int> * P_startstop) :
			ControlProc(pop),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class UserIface_1 : ParProc (PAR)
class UserIface_1 : public ParProc {
	private:
		SendChannel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "UserIface_1\\nPAR";}
		UserIface_1(Proc* pop, SendChannel<int> * P_startstop) :
			ParProc(pop),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class UserIface_1_1 : ControlProc (While)
class UserIface_1_1 : public ControlProc {
	private:
		SendChannel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "UserIface_1_1\\nWhile";}
		UserIface_1_1(Proc* pop, SendChannel<int> * P_startstop) :
			ControlProc(pop),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class UserIface_1_1_1 : ControlProc (New Scope)
class UserIface_1_1_1 : public ControlProc {
	private:
		SendChannel<int> * startstop;
		int* buttons;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "UserIface_1_1_1\\nNew Scope";}
		UserIface_1_1_1(Proc* pop, SendChannel<int> * P_startstop) :
			ControlProc(pop),
			startstop(take(P_startstop)),
			buttons(new int()) {}
};
// }}}
// {{{ Class UserIface_1_1_1_2 : CodeProc (Delete Scope)
class UserIface_1_1_1_2 : public CodeProc {
	private:
		SendChannel<int> * startstop;
		int* buttons;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "UserIface_1_1_1_2\\nDelete Scope";}
		UserIface_1_1_1_2(Proc* pop, SendChannel<int> * P_startstop, int* P_buttons) :
			CodeProc(pop),
			startstop(take(P_startstop)),
			buttons(P_buttons) {}
};
// }}}
// {{{ Class UserIface_1_1_1_1 : CodeProc (Assign)
class UserIface_1_1_1_1 : public CodeProc {
	private:
		SendChannel<int> * startstop;
		int* buttons;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void expand();
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "UserIface_1_1_1_1\\nAssign";}
		UserIface_1_1_1_1(Proc* pop, SendChannel<int> * P_startstop, int* P_buttons) :
			CodeProc(pop),
			startstop(take(P_startstop)),
			buttons(P_buttons) {}
};
// }}}
// {{{ Class UserIface_1_1_1_1_2 : CommProc (Comm)
class UserIface_1_1_1_1_2 : public CommProc {
	private:
		SendChannel<int> * startstop;
		int* buttons;
	public:
		virtual void transfer();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "UserIface_1_1_1_1_2\\nComm";}
		UserIface_1_1_1_1_2(Proc* pop, SendChannel<int> * P_startstop, int* P_buttons) :
			CommProc(pop, P_startstop, P_buttons),
			startstop(take(P_startstop)),
			buttons(P_buttons) {}
};
// }}}
// {{{ Class UserIface_1_2 : ControlProc (While)
class UserIface_1_2 : public ControlProc {
	private:
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "UserIface_1_2\\nWhile";}
		UserIface_1_2(Proc* pop) :
			ControlProc(pop) {}
};
// }}}
// {{{ Class UserIface_1_2_1 : ControlProc (Timer)
class UserIface_1_2_1 : public ControlProc {
	private:
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "UserIface_1_2_1\\nTimer";}
		UserIface_1_2_1(Proc* pop) :
			ControlProc(pop) {}
};
// }}}
// {{{ Class UserIface_1_2_1_1 : CodeProc ((expr))
class UserIface_1_2_1_1 : public CodeProc {
	private:
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "UserIface_1_2_1_1\\n(expr)";}
		UserIface_1_2_1_1(Proc* pop) :
			CodeProc(pop) {}
};
// }}}
// }}}

// {{{ Headers for process `Main'
// {{{ Class Main : ControlProc (Process)
class Main : public ControlProc {
	private:
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main\\nProcess";}
		Main(Proc* pop) :
			ControlProc(pop) {}
};
// }}}
// {{{ Class Main_1 : ControlProc (New Scope)
class Main_1 : public ControlProc {
	private:
		Channel<CODEDDATA> * coded;
		Channel<DACBUFFER> * decoded;
		Channel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1\\nNew Scope";}
		Main_1(Proc* pop) :
			ControlProc(pop),
			coded(new Channel<CODEDDATA> ()),
			decoded(new Channel<DACBUFFER> ()),
			startstop(new Channel<int> ()) {}
};
// }}}
// {{{ Class Main_1_2 : CodeProc (Delete Scope)
class Main_1_2 : public CodeProc {
	private:
		Channel<CODEDDATA> * coded;
		Channel<DACBUFFER> * decoded;
		Channel<int> * startstop;
	public:
		virtual int getExecTime() const {return 1; }
		virtual void run();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Main_1_2\\nDelete Scope";}
		Main_1_2(Proc* pop, Channel<CODEDDATA> * P_coded, Channel<DACBUFFER> * P_decoded, Channel<int> * P_startstop) :
			CodeProc(pop),
			coded(take(P_coded)),
			decoded(take(P_decoded)),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class Main_1_1 : ParProc (PAR)
class Main_1_1 : public ParProc {
	private:
		Channel<CODEDDATA> * coded;
		Channel<DACBUFFER> * decoded;
		Channel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return false;}
		virtual std::string getName() const {return "Main_1_1\\nPAR";}
		Main_1_1(Proc* pop, Channel<CODEDDATA> * P_coded, Channel<DACBUFFER> * P_decoded, Channel<int> * P_startstop) :
			ParProc(pop),
			coded(take(P_coded)),
			decoded(take(P_decoded)),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class Main_1_1_1 : ControlProc (Call)
class Main_1_1_1 : public ControlProc {
	private:
		SendChannel<CODEDDATA> * coded;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1_1_1\\nCall";}
		Main_1_1_1(Proc* pop, SendChannel<CODEDDATA> * P_coded) :
			ControlProc(pop),
			coded(take(P_coded)) {}
};
// }}}
// {{{ Class Main_1_1_2 : ControlProc (Call)
class Main_1_1_2 : public ControlProc {
	private:
		ReceiveChannel<CODEDDATA> * coded;
		SendChannel<DACBUFFER> * decoded;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1_1_2\\nCall";}
		Main_1_1_2(Proc* pop, ReceiveChannel<CODEDDATA> * P_coded, SendChannel<DACBUFFER> * P_decoded) :
			ControlProc(pop),
			coded(take(P_coded)),
			decoded(take(P_decoded)) {}
};
// }}}
// {{{ Class Main_1_1_3 : ControlProc (Call)
class Main_1_1_3 : public ControlProc {
	private:
		ReceiveChannel<DACBUFFER> * decoded;
		ReceiveChannel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1_1_3\\nCall";}
		Main_1_1_3(Proc* pop, ReceiveChannel<DACBUFFER> * P_decoded, ReceiveChannel<int> * P_startstop) :
			ControlProc(pop),
			decoded(take(P_decoded)),
			startstop(take(P_startstop)) {}
};
// }}}
// {{{ Class Main_1_1_4 : ControlProc (Call)
class Main_1_1_4 : public ControlProc {
	private:
		SendChannel<int> * startstop;
	public:
		virtual void expand();
		virtual bool isDiscovery() const {return true;}
		virtual std::string getName() const {return "Main_1_1_4\\nCall";}
		Main_1_1_4(Proc* pop, SendChannel<int> * P_startstop) :
			ControlProc(pop),
			startstop(take(P_startstop)) {}
};
// }}}
// }}}

// {{{ Source for process `DiskRead'
// {{{ Function `void DiskRead::expand()' (Process)
void DiskRead::expand() {
	connect(add(new DiskRead_1(pop, ch)));
}
// }}}
// {{{ Function `void DiskRead_1::expand()' (New Scope)
void DiskRead_1::expand() {
	Proc* pop = add(new DiskRead_1_2(this->pop, ch, buff));
	connect(add(new DiskRead_1_1(pop, ch, buff)));
}
// }}}
// {{{ Function `void DiskRead_1_2::run()' (Delete Scope)
void DiskRead_1_2::run() {
	delete buff;
}
// }}}
// {{{ Function `void DiskRead_1_1::expand()' (While)
void DiskRead_1_1::expand() {
	if (1) {
		Proc* loop = add(new DiskRead_1_1(pop, ch, buff));
		connect(add(new DiskRead_1_1_1(loop, ch, buff)));
	}
	else {
		connect(pop);
	}
}
// }}}
// {{{ Function `void DiskRead_1_1_1::expand()' (Assign)
void DiskRead_1_1_1::expand() {
	connect(add(new DiskRead_1_1_1_2(pop, ch, buff)));
}
// }}}
// {{{ Function `void DiskRead_1_1_1::run()' (Assign)
void DiskRead_1_1_1::run() {
	(*buff) = (buffLoadNext());
}
// }}}
// {{{ Function `void DiskRead_1_1_1_2::transfer()' (Comm)
void DiskRead_1_1_1_2::transfer() {
	ch->transfer();
}
// }}}
// }}}

// {{{ Source for process `Decode'
// {{{ Function `void Decode::expand()' (Process)
void Decode::expand() {
	connect(add(new Decode_1(pop, codedch, decodedch)));
}
// }}}
// {{{ Function `void Decode_1::expand()' (New Scope)
void Decode_1::expand() {
	Proc* pop = add(new Decode_1_2(this->pop, codedch, decodedch, coded, decoded));
	connect(add(new Decode_1_1(pop, codedch, decodedch, coded, decoded)));
}
// }}}
// {{{ Function `void Decode_1_2::run()' (Delete Scope)
void Decode_1_2::run() {
	delete coded;
	delete decoded;
}
// }}}
// {{{ Function `void Decode_1_1::expand()' (While)
void Decode_1_1::expand() {
	if (1) {
		Proc* loop = add(new Decode_1_1(pop, codedch, decodedch, coded, decoded));
		connect(add(new Decode_1_1_1(loop, codedch, decodedch, coded, decoded)));
	}
	else {
		connect(pop);
	}
}
// }}}
// {{{ Function `void Decode_1_1_1::expand()' (Comm)
void Decode_1_1_1::expand() {
	connect(add(new Decode_1_1_1_2(pop, codedch, decodedch, coded, decoded)));
}
// }}}
// {{{ Function `void Decode_1_1_1::transfer()' (Comm)
void Decode_1_1_1::transfer() {
	codedch->transfer();
}
// }}}
// {{{ Function `void Decode_1_1_1_2::expand()' (Assign)
void Decode_1_1_1_2::expand() {
	connect(add(new Decode_1_1_1_3(pop, codedch, decodedch, coded, decoded)));
}
// }}}
// {{{ Function `void Decode_1_1_1_2::run()' (Assign)
void Decode_1_1_1_2::run() {
	(*decoded) = (decode((*coded)));
}
// }}}
// {{{ Function `void Decode_1_1_1_3::transfer()' (Comm)
void Decode_1_1_1_3::transfer() {
	decodedch->transfer();
}
// }}}
// }}}

// {{{ Source for process `WriteDAC'
// {{{ Function `void WriteDAC::expand()' (Process)
void WriteDAC::expand() {
	connect(add(new WriteDAC_1(pop, ch, startstop)));
}
// }}}
// {{{ Function `void WriteDAC_1::expand()' (New Scope)
void WriteDAC_1::expand() {
	Proc* pop = add(new WriteDAC_1_2(this->pop, ch, startstop, decoded, cmd, play));
	connect(add(new WriteDAC_1_1(pop, ch, startstop, decoded, cmd, play)));
}
// }}}
// {{{ Function `void WriteDAC_1_2::run()' (Delete Scope)
void WriteDAC_1_2::run() {
	delete decoded;
	delete cmd;
	delete play;
}
// }}}
// {{{ Function `void WriteDAC_1_1::expand()' (While)
void WriteDAC_1_1::expand() {
	if (1) {
		Proc* loop = add(new WriteDAC_1_1(pop, ch, startstop, decoded, cmd, play));
		connect(add(new WriteDAC_1_1_1(loop, ch, startstop, decoded, cmd, play)));
	}
	else {
		connect(pop);
	}
}
// }}}
// {{{ Function `void WriteDAC_1_1_1::expand()' (Timer)
void WriteDAC_1_1_1::expand() {
	TimerEndProc* pop = add(new TimerEndProc(this->pop));
	Proc* next = add(new WriteDAC_1_1_1_1(pop, ch, startstop, decoded, cmd, play));
	connect(add(new TimerProc(next, pop, 10)));
}
// }}}
// {{{ Function `void WriteDAC_1_1_1_1::expand()' (Comm)
void WriteDAC_1_1_1_1::expand() {
	connect(add(new WriteDAC_1_1_1_1_2(pop, ch, startstop, decoded, cmd, play)));
}
// }}}
// {{{ Function `void WriteDAC_1_1_1_1::transfer()' (Comm)
void WriteDAC_1_1_1_1::transfer() {
	startstop->transfer();
}
// }}}
// {{{ Function `void WriteDAC_1_1_1_1_2::expand()' (If)
void WriteDAC_1_1_1_1_2::expand() {
		if (((*cmd)==(*play))) {
		connect(add(new WriteDAC_1_1_1_1_2_1(pop, ch, startstop, decoded, cmd, play)));
	}
		else
			connect(pop);
}
// }}}
// {{{ Function `void WriteDAC_1_1_1_1_2_1::expand()' (Comm)
void WriteDAC_1_1_1_1_2_1::expand() {
	connect(add(new WriteDAC_1_1_1_1_2_1_2(pop, ch, startstop, decoded, cmd, play)));
}
// }}}
// {{{ Function `void WriteDAC_1_1_1_1_2_1::transfer()' (Comm)
void WriteDAC_1_1_1_1_2_1::transfer() {
	ch->transfer();
}
// }}}
// {{{ Function `void WriteDAC_1_1_1_1_2_1_2::run()' ((expr))
void WriteDAC_1_1_1_1_2_1_2::run() {
	(writeDAC((*decoded)));
}
// }}}
// }}}

// {{{ Source for process `UserIface'
// {{{ Function `void UserIface::expand()' (Process)
void UserIface::expand() {
	connect(add(new UserIface_1(pop, startstop)));
}
// }}}
// {{{ Function `void UserIface_1::expand()' (PAR)
void UserIface_1::expand() {
	ParEndProc* join = add(new ParEndProc(pop, this));
	connect(add(new UserIface_1_1(join, startstop)));
	connect(add(new UserIface_1_2(join)));
}
// }}}
// {{{ Function `void UserIface_1_1::expand()' (While)
void UserIface_1_1::expand() {
	if (1) {
		Proc* loop = add(new UserIface_1_1(pop, startstop));
		connect(add(new UserIface_1_1_1(loop, startstop)));
	}
	else {
		connect(pop);
	}
}
// }}}
// {{{ Function `void UserIface_1_1_1::expand()' (New Scope)
void UserIface_1_1_1::expand() {
	Proc* pop = add(new UserIface_1_1_1_2(this->pop, startstop, buttons));
	connect(add(new UserIface_1_1_1_1(pop, startstop, buttons)));
}
// }}}
// {{{ Function `void UserIface_1_1_1_2::run()' (Delete Scope)
void UserIface_1_1_1_2::run() {
	delete buttons;
}
// }}}
// {{{ Function `void UserIface_1_1_1_1::expand()' (Assign)
void UserIface_1_1_1_1::expand() {
	connect(add(new UserIface_1_1_1_1_2(pop, startstop, buttons)));
}
// }}}
// {{{ Function `void UserIface_1_1_1_1::run()' (Assign)
void UserIface_1_1_1_1::run() {
	(*buttons) = (readButtons());
}
// }}}
// {{{ Function `void UserIface_1_1_1_1_2::transfer()' (Comm)
void UserIface_1_1_1_1_2::transfer() {
	startstop->transfer();
}
// }}}
// {{{ Function `void UserIface_1_2::expand()' (While)
void UserIface_1_2::expand() {
	if (1) {
		Proc* loop = add(new UserIface_1_2(pop));
		connect(add(new UserIface_1_2_1(loop)));
	}
	else {
		connect(pop);
	}
}
// }}}
// {{{ Function `void UserIface_1_2_1::expand()' (Timer)
void UserIface_1_2_1::expand() {
	TimerEndProc* pop = add(new TimerEndProc(this->pop));
	Proc* next = add(new UserIface_1_2_1_1(pop));
	connect(add(new TimerProc(next, pop, 45)));
}
// }}}
// {{{ Function `void UserIface_1_2_1_1::run()' ((expr))
void UserIface_1_2_1_1::run() {
	(updatedisplay());
}
// }}}
// }}}

// {{{ Source for process `Main'
// {{{ Function `void Main::expand()' (Process)
void Main::expand() {
	connect(add(new Main_1(pop)));
}
// }}}
// {{{ Function `void Main_1::expand()' (New Scope)
void Main_1::expand() {
	Proc* pop = add(new Main_1_2(this->pop, coded, decoded, startstop));
	connect(add(new Main_1_1(pop, coded, decoded, startstop)));
}
// }}}
// {{{ Function `void Main_1_2::run()' (Delete Scope)
void Main_1_2::run() {
	delete coded;
	delete decoded;
	delete startstop;
}
// }}}
// {{{ Function `void Main_1_1::expand()' (PAR)
void Main_1_1::expand() {
	ParEndProc* join = add(new ParEndProc(pop, this));
	connect(add(new Main_1_1_1(join, makeSend(coded))));
	connect(add(new Main_1_1_2(join, makeReceive(coded), makeSend(decoded))));
	connect(add(new Main_1_1_3(join, makeReceive(decoded), makeReceive(startstop))));
	connect(add(new Main_1_1_4(join, makeSend(startstop))));
}
// }}}
// {{{ Function `void Main_1_1_1::expand()' (Call)
void Main_1_1_1::expand() {
	connect(add(new DiskRead(pop, coded)));
}
// }}}
// {{{ Function `void Main_1_1_2::expand()' (Call)
void Main_1_1_2::expand() {
	connect(add(new Decode(pop, coded, decoded)));
}
// }}}
// {{{ Function `void Main_1_1_3::expand()' (Call)
void Main_1_1_3::expand() {
	connect(add(new WriteDAC(pop, decoded, startstop)));
}
// }}}
// {{{ Function `void Main_1_1_4::expand()' (Call)
void Main_1_1_4::expand() {
	connect(add(new UserIface(pop, startstop)));
}
// }}}
// }}}

