#ifndef __INCLUDE_PARPROC_HPP__
#define __INCLUDE_PARPROC_HPP__

#include "Proc.hpp"

#include <string>

class ParProc : public Proc {
	private:
		ProcList children;
		ProcIt it;

	public:
		ParProc(Proc* pop) : Proc(pop) {}

		virtual Proc* connect(Proc* child) {
			assert(child);
			children.push_back(child);
			it = children.end();
			child->addParent();

			return child;
		}

		// Return count of children. 
		virtual size_t countChildren() const {
			return children.size();
		}

		virtual Proc* getChild() {
			assert(!"Cannot call `getChild()` on PAR statement (must call `getChildren()`)");
			return NULL;
		}

		// getNextChild circulates through all children sequentially.
		virtual Proc* getNextChild() {
			if (children.empty())
				return NULL;
			else {
				if (it == children.end()) {
					it = children.begin();
				}

				return *it++;
			}
		}

		const ProcList& getChildren() const {
			return children;
		}

		ProcList& getChildren() {
			return children;
		}

		virtual std::string getDotExtra() const {
			std::string extra;
			for (ProcConstIt it = children.begin(); 
					it != children.end(); ++it) {
				extra += getDotDescription() + " -> " + (*it)->getDotDescription() + ";\n";
			}
			return extra;
		}

};

#endif //#ifndef __INCLUDE_PARPROC_HPP__

