#ifndef __INCLUDE_SCHEDULER_HPP__
#define __INCLUDE_SCHEDULER_HPP__

class TimerProc;
class TimerEndProc;
class CommProc;
class ControlProc;
class CodeProc;
class ParProc;
class ParEndProc;
class AltProc;
class Proc;

#include <deque>
#include <list>

enum ProcState {CLOSED, EXPANDED, SERIALIZED, DELETABLE, DELETED};

typedef std::list<Proc*> ProcList;
typedef std::list<Proc*>::const_iterator ProcConstIt;
typedef std::list<Proc*>::iterator ProcIt;

class Scheduler {
	private:
		ProcList procs;
		std::deque<Proc*> serialized;
		std::list<TimerProc*> timers;

		int now;
		int predictionTime;

		enum SerializeResult {CONTINUE, RESELECT, COMPLETED, UNCERTAIN, OVERFLOW, PROGRAMFINISHED};

		Scheduler() : now(0), predictionTime(0) {}

		void print(bool printSerialized);
		void doAddTimer(TimerProc* timer, int startTime);
		void doRemoveTimer(TimerProc* timer);
		void doSerialize(Proc* t);
		void doExpand(Proc* t);

		void serialize(TimerProc* active, int horizon, SerializeResult* result);
		Proc* serializeTimerProc(TimerProc* timer, SerializeResult* result);
		Proc* serializeTimerEndProc(TimerEndProc* timerEnd, SerializeResult* result);
		Proc* serializeControlProc(ControlProc* ctl, SerializeResult* result);
		Proc* serializeCommProc(CommProc* comm, SerializeResult* result);
		Proc* serializeParProc(ParProc* par, SerializeResult* result);
		Proc* serializeParEndProc(ParEndProc* parEnd, SerializeResult* result);
		Proc* serializeCodeProc(CodeProc* code, int* horizon, SerializeResult* result);
		Proc* serializeAltProc(AltProc* alt, SerializeResult* result);
		void select(int horizon);
		void discover(Proc* t, int time);
	public:
		static Scheduler& Instance() {
			static Scheduler instance;
			return instance;
		}

		Proc* add(Proc* t);
		Proc* remove(Proc* t);

		void main();
};


#endif //#ifndef __INCLUDE_SCHEDULER_HPP__

