#ifndef __INCLUDE_TIMERPROC_HPP__
#define __INCLUDE_TIMERPROC_HPP__

#include "Proc.hpp"
#include "TimerEndProc.hpp"

#include <string>
#include <cassert>

class TimerProc : public Proc {
	private:
		const int time;
		int startTime;
		Proc* next;
		TimerEndProc* timerEnd;
	public:
		TimerProc(Proc* next, TimerEndProc* timerEnd, int time) :
				Proc(NULL), time(time), startTime(-1), next(next),
				timerEnd(timerEnd) {
			timerEnd->setTimerProc(this);
		}

		void setNext(Proc* next) {
			assert(getChild() != NULL);
			this->next = next;
		}

		Proc* getNext() const {
			return next;
		}

		int getTime() const {
			return time;
		}

		int getStartTime() const {
			return startTime;
		}

		void setStartTime(int startTime) {
			this->startTime = startTime;
		}

		virtual std::string getDotStyle() const { 
			if (getState() == SERIALIZED && 
					timerEnd->getState() == SERIALIZED) {
				return "[shape=invtriangle, style=filled, fillcolor=greenyellow]";
			}
			else if (getState() == SERIALIZED) {
				return "[shape=invtriangle, style=filled, fillcolor=yellow]";
			}
			else {
				return "[shape=invtriangle]";
			}
		}

		virtual std::string getName() const {
			return toStr(time);
		}

		virtual std::string getDotExtra() const {
			assert(next);
			assert(timerEnd);

			std::string extra;
			extra += getDotDescription() + " -> " + timerEnd->getDotDescription() +
				"[dir=none,label=\"pair\",style=dotted];\n";
			
			extra += getDotDescription() + " -> " + next->getDotDescription() +
				"[style=dashed,label=\"next\"];\n";

			return extra;
		}

		virtual void expand() {
			connect(next);
		}

};

struct DeadlineCompare {
	bool operator()(const TimerProc* lhs, const TimerProc* rhs) {
		return lhs->getTime() + lhs->getStartTime() 
			< rhs->getTime() + rhs->getStartTime();
	}
};


#endif //#ifndef __INCLUDE_TIMERPROC_HPP__

